var services = [];

function addService(srv){
    srv.initialize(services.length);
    services.push(srv);
}


Function.prototype.bind = function() {
    // utility
    function $A(iterable) {
	if (!iterable) return [];
	if ('toArray' in Object(iterable)) return iterable.toArray();
	var length = iterable.length || 0, results = new Array(length);
	while (length--) results[length] = iterable[length];
	return results;
    }
    var __method = this, args = $A(arguments), object = args.shift();
    return function() {
	return __method.apply(object, args.concat($A(arguments)));
    }
};

function requestFile( fileName, async, on_complete )
{
    var httpoj = new XMLHttpRequest();
    httpoj.open( "GET" , fileName , async );
    httpoj.setRequestHeader("If-Modified-Since", "Thu, 01 Jun 1970 00:00:00 GMT");
    httpoj.onreadystatechange = function() {
	if (httpoj.readyState==4) {
	    on_complete(httpoj);
	}
    };
    httpoj.send(null);
}


function Service() {
    this.initialize = function(_index){
	this.index = _index;
    };
    this.getChannelId = function(name){
	return chs[name][this.index];
    };
    // channel status setter/getter
    this.getStatus = function(name){
	return ( ch_status[name] & (1<<this.index) ) > 0;
    };
    this.setStatus = function(name){
	ch_status[name] |= (1<<this.index);
    };
    this.unsetStatus = function(name, index){
	ch_status[name] &= ~(1<<this.index);
    };
    // loading status counter
    this.ch_loading = new Object();
    this.incLoading = function(name){
	if(!this.ch_loading[name]) this.ch_loading[name] = 0;
	this.ch_loading[name]++;
    };
    this.decLoading = function(name){
	this.ch_loading[name]--;
    };
    // error status
    this.ch_error = new Object();
    this.getError = function(name){
	return ( this.ch_error[name] & (1<<this.index) ) > 0;
    };
    this.setError = function(name){
	this.ch_error[name] |= (1<<this.index);
    };
    this.unsetError = function(name){
	this.ch_error[name] &= ~(1<<this.index);
    };
    this.getStatusClass = function(name){
	return (ch_status[name] > 0 ? 'online' : 'offline') + (this.ch_loading[name] > 0 ? ' loading' : '') +  (this.ch_error[name] > 0 ? ' error' : '');
    };


    this.checkStatus = function(name, url, func, iserr){
	this.incLoading(name);
	updateFlyoutStatus(name);
	requestFile(url, true,
		    function(oj){
			var res  = oj.responseText;
			var live = func(res);
			this.unsetError(name);
			if(live)
			  this.setStatus(name, this.index);
			else{
			    this.unsetStatus(name, this.index);
			    if(iserr(res)) this.setError(name);
			}
			this.decLoading(name);
			updateFlyoutStatus(name);
		    }.bind(this));
    };
    this.checkStatusWithHTML = function(name, url, match_text, valid_text){
	this.checkStatus(name, url, function(res){return res.match(match_text);},
			 function(res){return !res.match(valid_text);});
    };
    this.availableChannels = [];
    this.nameReverse = [];
    this.prepare = function(){
	var ids = [];
	var rev = new Object();
	for(var name in chs){
	    this.unsetError(name);
	    var id = this.getChannelId(name);
	    if(id){
		ids.push(id);
		rev[id] = name;
	    }
	}
	this.availableChannels[this.index] = ids;
	this.nameReverse[this.index] = rev;
    };
    this.getNameFromId = function(ch){ return this.nameReverse[this.index][ch]; };
    this.getAvailableChannels = function(){ return this.availableChannels[this.index]; };

    // require implement in subclass (this is dummy)
    this.name = '';                                 // service name (display in config)
    this.displayContent = function(ch_arr, w, h){}; // return display content
    this.renewStatus = function(chs){};             // update lives tatus
};
var basicService = new Service();

// Ustream
function UstreamService(){
    this.name = "Ustream";
    this.ustChIds = new Object();
    this.setUstId = function(ch, id){
	this.ustChIds[ch] = id;
    };

    this.getUstId = function(ch){
	return this.ustChIds[ch];
    };

    this.getUstreamChannelId = function(ch) {
	if(!this.getUstId(ch)) {
	    requestFile('http://www.ustream.tv/channel/' + ch, false,
			function(oj){
			    var id;
			    var res  = oj.responseText;
			    id = res.match('channelid=([0-9]+)');
			    if(id)
			      this.setUstId(ch, id[1]);
			    else
			      if(id = res.match('rel="([0-9]+)"'))
				this.setUstId(ch, id[1]);
			}.bind(this));

	    if(!this.getUstId(ch) || this.getUstId(ch)=='') // if fail, use api
	      requestFile('http://api.ustream.tv/xml/channel/' +
			  encodeURIComponent(ch) +
			  '/getValueOf/id',
			  false,
			  function(oj) {
			      var id = oj.responseXML.selectSingleNode('/xml/results').text;
			      this.setUstId(ch, id);
			  }.bind(this));

	    if(this.getUstId(ch) == '')
	      this.setError(this.getNameFromId(ch));
	    else
	      this.unsetError(this.getNameFromId(ch));
	}
	return this.getUstId(ch);
    };

    this.displayContent = function(ch_arr, w, h){
	var ch = encodeURI(ch_arr[this.index]);
	var chid = this.getUstreamChannelId(ch);
	if(!chid)chid = '1';
	var url = 'http://www.ustream.tv/flash/live/1/' + chid;

	return '<object id="movie" width="'+w+'" height="'+h
	  + '" type="application/x-shockwave-flash" classid="clsid:D27CDB6E-AE6D-11cf-96B8-444553540000">'
	  + '<param name="movie" value="' + url
	  + '"/><param name="flashvars" value="viewcount=true&amp;autoplay='
	  + (autoplay?'true':'false')+'&amp;brand=embed"/>'
	  + '<param name="wmode" value="'+ getFlashMode() +'"/>'
	  + '<param name="allowFullScreen" value="true"/>'
	  + '<param name="allowscriptaccess" value="always"/></object>';
    };
    //this.names = new Object();
    this.renewStatus = function(chs){
	var tmp = this.getAvailableChannels();
	var err = 0;
	var len = tmp.length;
	for (var i = 0; i < len; i += 10){
	    var curr = tmp.slice(i,i+10);
	    //Debug.writeLine(curr.join('/'));
	    if(curr.length == 1){
		this.checkStatus(this.getNameFromId(curr[0]),
				 'http://api.ustream.tv/json/channel/'+encodeURI(curr[0])+'/getValueOf/status',
				 function(res){
				     eval('var a = '+res);
				     if(!a || !a.results){
					 this.fallbackCheck(curr);
					 return;
				     }
				     if(a.results.match('live'))
				       this.setStatus(this.getNameFromId(curr[0]));
				     else
				       this.unsetStatus(this.getNameFromId(curr[0]));
				     updateFlyoutStatus(this.getNameFromId(curr[0]));
				 }.bind(this),
				 function(){return false;});
	    }else if(curr.length > 1){
		this.checkStatus('Revin', // dummy, TODO
				 'http://api.ustream.tv/json/channel/'+encodeURI(curr.join(';'))+'/getValueOf/status',
				 function (ids,obj){
				     return function(res){
					 eval('results = '+res);
					 if(!results || results.error){
					     this.fallbackCheck(ids);
					     return;
					 }
					 for(var i =0; i < results.results.length; i++){
					     var a = results.results[i];
					     var tn = this.getNameFromId(a.uid);
					     if(a.result.match('live'))
					       this.setStatus(tn);
					     else
					       this.unsetStatus(tn);
					     updateFlyoutStatus(tn);
					 }
				     }.bind(obj);
				 }(curr,this),
				 function(){return false;});
	    }else{
		;//err
	    }
	}
    };
    this.fallbackCheck = function(ids) {
	var len = ids.length;
	for(var i = 0; i < len; i++){
	    var x = ids[i];
	    var name = this.getNameFromId(x);
	    this.checkStatusWithHTML(this.getNameFromId(x),
				     'http://api.ustream.tv/json/channel/'+encodeURI(x)+'/getValueOf/status',
				     'live',
				     '"error":null');
	}
    };
};
UstreamService.prototype = basicService;


// Justin
function JustinService()
{
    this.name = "Justin";
    this.displayContent = function(ch_arr, w, h){
	var ch = encodeURI(ch_arr[this.index]);

	if( jstHack )
	  return '<iframe align="center" name="movie" id="movie"'
	    + 'src="http://www.justin.tv/widgets/clip_embed_player.swf?&amp;channel='+ch
	    + '&amp;tip_id=180622" width="'+w+'" height="'+h+'" frameborder="0"/>';
	else
	  return '<object type="application/x-shockwave-flash" height="'+h+'" width="'+w+'"'
	    + 'id="movie" data="http://www.justin.tv/widgets/live_embed_player.swf?channel='
	    + ch + (jstAPI? '&amp;consumer_key=hQ3kBxqRXXBZJlPmu03JnA' : '') + '" bgcolor="#000000">'
//	    + ch +'&amp;consumer_key=hQ3kBxqRXXBZJlPmu03JnA" bgcolor="#000000">'
	    + '<param name="allowFullScreen" value="true" />'
	    + '<param name="allowScriptAccess" value="always" />'
	    + '<param name="allowNetworking" value="all" />'
	    + '<param name="wmode" value="'+ getFlashMode() +'"/>'
	    + '<param name="movie" value="http://www.justin.tv/widgets/live_embed_player.swf" />'
	    + '<param name="flashvars" value="channel='+ ch +'&amp;auto_play='
	    + (autoplay ? 'true' : 'false') +'&amp;start_volume=0.25'+  (jstAPI? '&amp;consumer_key=hQ3kBxqRXXBZJlPmu03JnA' : '') +'" />'
	    + '</object>';
    };

    this.renewStatus = function(chs){
	var tmp = this.getAvailableChannels();
	var len = tmp.length;
	for(var i=0; i < len; i++ ){
	    this.checkStatusWithHTML(this.getNameFromId(tmp[i]),
				     "http://www.justin.tv/"+encodeURI(tmp[i]),
				     "PP\\['channel_live'\\] = true;",
				     "PP\\['channel'\\]");
	}
    };
}
JustinService.prototype = basicService;


// Blogtv
function BlogtvService()
{
    this.name = "blogTV";
    this.btvChInfos = new Object();
    this.setBtvInfo = function(ch, info){
	this.btvChInfos[ch] = info;
    };
    this.getBtvInfo = function(ch){
	return this.btvChInfos[ch];
    };
    this.getBlogtvChannelInfo = function(ch) {
	if(!this.getBtvInfo(ch)) {
	    requestFile('http://www.blogtv.com/people/' + ch, false,
			function(oj){
			    var mat;
			    var res  = oj.responseText;
			    var obj = new Object();
			    if(mat = res.match('var _liveEncEmbed="([0-9a-zA-Z]+)"')) obj.liveEnc = mat[1];
			    if(mat = res.match('var _OwnerProgramId=([0-9]+)')) obj.programId = mat[1];
			    if(mat = res.match('var _OwnerUserId=([0-9]+)')) obj.userId = mat[1];
			    if(mat = res.match('var _FMServerName *= *"([0-9a-z\.]+)"')) obj.serverName = mat[1];
			    if(mat = res.match('var _EdgeFMServerName *= *"([0-9a-z\.]+)"')) obj.edgeServerName = mat[1];
			    if(mat = res.match('var _channelId = "([0-9]+)"')) obj.channelId = mat[1];
			    this.setBtvInfo(ch, obj);
			}.bind(this));
	    if(!this.getBtvInfo(ch) || !this.getBtvInfo(ch).liveEnc)
	      this.setError(this.getNameFromId(ch));
	    else
	      this.unsetError(this.getNameFromId(ch));
	}
	return this.getBtvInfo(ch);
    };
    this.displayContent = function(ch_arr, w, h){
	var ch = encodeURI(ch_arr[this.index]);
	var info = this.getBlogtvChannelInfo(ch);
	if( btvExp )
	  return '<iframe align="center" name="movie" id="movie" src="http://www.blogtv.com/Flash/BTP.swf?iniLocation=http://www.blogtv.com/en/FXml/&liveEnc='
	    + info.liveEnc+'&ApplicationName=BlogTV&ServerName='+info.serverName+'&EdgeServerName='+info.edgeServerName
	      + '&BroadCasterUserid='+info.userId+'&ProgId='+info.programId+'&PlayerName='+ch+'&channelId='+info.channelId
		+ '&chDescUrlAd=http://www.blogtv.com/people/'+ch+'&durationAd=2700&ps=0&ut=0&ts=0&fme=1" width="'+w+'" height="'+h+'" frameborder="0"/>';
	//return '<iframe src="http://www.blogtv.com/People/'+ch+'" width="'+w+'" height="'+h+'" scroll="no" marginheight="100" marginwidth="100" />';
	else
	  /*
		  // official embed swf   
		  return '<object id="movie" width="'+ w +'" height="'+ h +'" type="application/x-shockwave-flash"'
			+ ' data="http://www.blogtv.com/livesdk/' + info.liveEnc + '">'
			+ '<param name="movie" value="http://www.blogtv.com/livesdk/' + info.liveEnc + '"/>'
			+ '<param name="wmode" value="'+ getFlashMode() +'"/>'
			+ '<param name="allowFullScreen" value="true"/></object>';
			*/
	  // official site top swf
	  return '<object id="movie" width="'+ w +'" height="'+ h +'" type="application/x-shockwave-flash">'
	    + '<param name="movie" value="http://www.blogtv.com/Flash/livePlayerHP.swf?iniLocation=http://www.blogtv.com/en/FXml/&liveEnc=' + info.liveEnc + '"/>'
	      + '<param name="wmode" value="'+ getFlashMode() +'"/>'
		+ '<param name="allowFullScreen" value="true"/></object>';
    };

    this.renewStatus = function(chs){
	var tmp = this.getAvailableChannels();
	var len = tmp.length;
	for(var i=0; i < len; i++ ){
	    var ch = encodeURI(tmp[i]);
	    this.checkStatusWithHTML(this.getNameFromId(tmp[i]),
				     "http://www.blogtv.com/Search.asp?SearchType=0&SearchStr="+ch,
				     "<table.*\'/People/"+ch+"\'.*http://storage\.blogtv\.com/Images/en/LIVE\.gif.*</table>",
				     "<table.*\'/People/"+ch+"\'");
	    /*
		var info = this.getBlogtvChannelInfo(tmp[i]);
		this.checkStatusWithHTML(this.getNameFromId(tmp[i]),
								 "http://www.blogtv.com/Ajax/Program_ProgramIsLive.asp?UId="+info.userId,			
								 '<isLive><\\!\\[CDATA\\[1\\]\\]></isLive>',									 
								 '<fms><\\!\\[CDATA\\[fms');									 
							 */
	}
    };
};
BlogtvService.prototype = basicService;


addService(new UstreamService());
addService(new JustinService());
addService(new BlogtvService());

